@@ -6,9 +6,9 @@ from senex_shop.checkout.views import GatewayView, ShippingAddressView, Shipping
 urlpatterns = patterns('',
                        url(r'^$', GatewayView.as_view(), name='checkout_start'),
                        url(r'^shipping-address/$', ShippingAddressView.as_view(), name='checkout_shipping_address'),
-                       url(r'^shipping-address/$', UserAddressUpdateView.as_view(),
+                       url(r'^shipping-address/edit/(?P<pk>\d+)/$', UserAddressUpdateView.as_view(),
                            name='checkout_user_address_update'),
-                       url(r'^shipping-address/$', UserAddressDeleteView.as_view(),
+                       url(r'^shipping-address/delete/(?P<pk>\d+)/$', UserAddressDeleteView.as_view(),
                            name='checkout_user_address_delete'),
                        url(r'^shipping-method/$', ShippingMethodView.as_view(), name='checkout_shipping_method'),
                        url(r'^payment-method/$', PaymentMethodView.as_view(), name='checkout_payment_method'),
